/*******************************************************************************
 * Copyright (c) 2005 BEA Systems, Inc. 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.example.processor.internal;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import com.sun.mirror.apt.AnnotationProcessor;
import com.sun.mirror.apt.AnnotationProcessorEnvironment;
import com.sun.mirror.apt.AnnotationProcessorFactory;
import com.sun.mirror.declaration.AnnotationTypeDeclaration;

/**
 * A factory object, with a zero-argument constructor, that
 * knows how to construct new AnnotationProcessor instances.
 * This object will be constructed by APT, either via reflection
 * or via the annotationProcessorFactories extension point.
 */
public class APFactory implements AnnotationProcessorFactory {

	// Set to true in order to activate debug tracing.
	public static boolean TRACING = false;
	
	// Constants used for tracing
	private static final SimpleDateFormat TRACE_DATE_FORMAT = new SimpleDateFormat("HH:mm:ss.SSS"); //$NON-NLS-1$

	// To turn debug tracing on when running inside apt, add this option to the command line.
	private static final Object TRACE_OPTION = "-Aorg.example.processor/debug";
	
	// TODO: specify the annotations this processor supports
	private static final String[] _supportedAnnotations = { };

	/**
	 * Send a message to the debug console, marked with timestamp and thread ID.
	 * To enable this output it is necessary to turn tracing on.  When running
	 * in the Eclipse debugger this can be done via the 'tracing' tab on the
	 * launch configuration dialog; when launching Eclipse from the command line
	 * it can be done by specifying a .options file with the -debug command line.
	 * <p>
	 * More information is available at 
	 * <a href="http://wiki.eclipse.org/index.php/FAQ_How_do_I_use_the_platform_debug_tracing_facility%3F">
	 * Eclipsepedia</a>
	 * @param msg
	 */
	public static void trace(final String msg){
		if (TRACING) {
			StringBuffer sb = new StringBuffer();
			sb.append('[');
			// SimpleDateFormat is not thread-safe, according to javadoc
			synchronized(TRACE_DATE_FORMAT) {
				sb.append(TRACE_DATE_FORMAT.format(new Date()));
			}
			sb.append('-');
			// Some threads have qualified type names; too long.
			String threadName = Thread.currentThread().getName();
			int dot = threadName.lastIndexOf('.');
			if (dot < 0) {
				sb.append(threadName);
			}
			else {
				sb.append(threadName.substring(dot+1));
			}
			sb.append(']');
			sb.append(msg);
			System.out.println(sb);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public AnnotationProcessor getProcessorFor(
			Set<AnnotationTypeDeclaration> atds,
			AnnotationProcessorEnvironment env) {
		if (env.getOptions().keySet().contains(TRACE_OPTION)) {
			TRACING = true;
		}
		return new Processor(atds, env);
	}

	/**
	 * {@inheritDoc}
	 */
	public Collection<String> supportedAnnotationTypes() {
		if (_supportedAnnotations.length == 0) {
			return null;
		}
		Set<String> supportedAnnotations = new HashSet<String>(_supportedAnnotations.length);
		for (String annotation : _supportedAnnotations) {
			supportedAnnotations.add(annotation);
		}
		return supportedAnnotations;
	}

	/**
	 * {@inheritDoc}
	 */
	public Collection<String> supportedOptions() {
		//TODO: optionally, specify the -A options supported by this processor.
		return null;
	}

}
