/*******************************************************************************
 * Copyright (c) 2007 BEA Systems, Inc. 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package org.example.processor.internal.utils;

import java.util.Map;

import com.sun.mirror.declaration.AnnotationMirror;
import com.sun.mirror.declaration.AnnotationTypeDeclaration;
import com.sun.mirror.declaration.AnnotationTypeElementDeclaration;
import com.sun.mirror.declaration.AnnotationValue;
import com.sun.mirror.declaration.Declaration;
import com.sun.mirror.declaration.TypeDeclaration;
import com.sun.mirror.type.AnnotationType;

public class DeclarationHelper {

	/**
	 * One way to get a type's package name is to call
	 * {@link TypeDeclaration#getPackage} and then
	 * {@link PackageDeclaration#getQualifiedName}. However, creating a
	 * <code>PackageDeclaration</code> is very expensive, so the
	 * <code>getPackage</code> call is expensive. It is more efficient to do
	 * it by parsing the type's name; this is a helper routine to encapsulate
	 * that task.
	 * 
	 * @param decl
	 *            the type whose package is being queried. May be a nested or
	 *            top-level type.
	 * @return the qualified name of the package, delimited with '.', or the
	 *         empty string if the declaration is in the default package.
	 */
	public static String getPackageName(TypeDeclaration decl) {
		TypeDeclaration parent;
		while ((parent = decl.getDeclaringType()) != null) {
			decl = parent;
		}
		String topClass = decl.getQualifiedName();
		int lastDot = topClass.lastIndexOf('.');
		if (lastDot < 1) {
			return "";
		}
		return topClass.substring(0, lastDot);
	}

	/**
	 * Find a particular annotation on a declaration
	 * 
	 * @param annoName
	 *            the qualified name of the annotation to search for
	 * @param decl
	 *            the possibly-annotated declaration
	 * @return an annotation mirror representing the annotation, or null if the
	 *         annotation was not found.
	 */
	public static AnnotationMirror getAnnotationMirror(String annoName,
			Declaration decl) {
		if (null == decl || null == annoName || annoName.length() == 0) {
			return null;
		}
		for (AnnotationMirror anno : decl.getAnnotationMirrors()) {
			AnnotationType type = anno.getAnnotationType();
			if (type == null) {
				continue;
			}
			AnnotationTypeDeclaration annoDecl = type.getDeclaration();
			if (annoDecl == null) {
				continue;
			}
			if (annoName.equals(annoDecl.getQualifiedName())) {
				return anno;
			}
		}
		return null;
	}

	/**
	 * Find the value of a particular element of an annotation. An "element" is
	 * an annotation parameter; e.g., in <code>@Anno(foo="bar")</code> it is <code>foo</code>. They're called
	 *                  elements instead of parameters because technically they
	 *                  are method declarations, just like technically an
	 *                  annotation is a kind of interface.
	 * 
	 * @param elementName
	 *            the name of the element, or null for the default element
	 *            "value"
	 * @param anno
	 *            the annotation mirror to search
	 * @return an AnnotationValue representing the value, or null if the element
	 *         was not found. Note that an AnnotationValue is itself a wrapper
	 *         around the actual value.
	 */
	public static AnnotationValue getAnnotationValue(String elementName,
			AnnotationMirror anno) {
		if (null == elementName || elementName.length() < 1 || null == anno) {
			return null;
		}
		Map<AnnotationTypeElementDeclaration, AnnotationValue> values = anno
				.getElementValues();
		for (Map.Entry<AnnotationTypeElementDeclaration, AnnotationValue> entry : values
				.entrySet()) {
			if (elementName.equals(entry.getKey().getSimpleName())) {
				return entry.getValue();
			}
		}
		return null;
	}
}
